#!/usr/bin/env python
#
#
#   Copyright (c) 2007 Jesse Guardiani <jesse@guardiani.us>
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation; either version 2 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
#   02111-1307, USA.
#


from manager import manager as manager


class BasePlayer:
  ''' This class defines the public API for every player derived from it 
  and also sets some sane defaults and basic functionality. '''
  paused = False
  volume = 100
  target = None
  
  def __init__(self):
    self.percent   = 0
    self.seconds   = 0.0
    self.length    = 0.0
    self.powersave = False
    self._extended_init()
    self.set_volume(manager.prefs.getInt("volume"))

  def _extended_init(self):
    ''' do init stuff '''
    ''' override me '''

  def _update_volume(self):
    ''' set player volume '''
    ''' override me '''

  def _update_pause(self):
    ''' set paused/playing '''
    ''' override me '''

  def _update_play(self):
    ''' set play target '''
    ''' override me '''

  def _update_powersave(self):
    ''' set powersave state on/off '''
    ''' override me '''

  ''' This method should be overridden to hook VOIP interruptions '''
  def _interrupt(self, begin):
    self.pause(begin)
    ''' override me if needed '''

  def play(self, target):
    ''' play a file '''
    self.length  = 0.0
    self.percent = 0
    self.seconds = 0.0
    self.target  = target
    self.paused  = False
    self._update_play()

  def volume_increase(self):
    self.volume += 10
    if self.volume > 100: self.volume = 100
    self._update_volume()
    manager.show_volume()

  def volume_decrease(self):
    self.volume -= 10
    if self.volume < 0: self.volume = 0
    self._update_volume()
    manager.show_volume()
    
  def set_volume(self,new_volume):
    if new_volume < 0: new_volume = 0
    if new_volume > 100: new_volume = 100
    self.volume = new_volume
    self._update_volume()
    manager.show_volume()
    
  def pause(self, is_paused = None):
    if is_paused == None:
      if self.paused: self.paused = False
      else: self.paused = True
    else: self.paused = is_paused
    self._update_pause()
    manager.play_button.update()
    
  def seek(self, amount, mode=0):
    ''' seek '''

  def set_powersave(self, is_powersave):
    if self.powersave == is_powersave: return # only call for changes in state
    self.powersave = is_powersave
    print "powersave = %s" % (self.powersave,)
    self._update_powersave()
  
  def close(self):
    ''' close '''
