#!/usr/bin/env python
#
#
#   Copyright (c) 2007 Jesse Guardiani <jesse@guardiani.us>
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation; either version 2 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
#   02111-1307, USA.
#

import os, sqlite3
import globals


class DB():
  db_dir = None
  path = None
  con = None
  c = None

  def __init__(self):
    self.db_dir = globals.calc_db_dir()
    self.path = os.path.join(self.db_dir,'kagu.db')
    self.connect()

  def reconnect(self):
    self.close()
    self.connect()

  def connect(self):
    self.con = sqlite3.connect(self.path)
    self.con.row_factory = sqlite3.Row
    self.c   = self.con.cursor()
    self.c.execute('PRAGMA synchronous = OFF;')

  def close(self):
    self.con.close()
    self.con = None
    self.c   = None
    
  def get_db_dir(self):
    return self.db_dir

  def artist_of_album(self,album):
    self.c.execute('''
      SELECT al.name AS album
           , ar.name AS artist
           , al.art_path AS art_path
        FROM song s
             JOIN album  al ON s.album_id  = al.id
             JOIN artist ar ON s.artist_id = ar.id
       WHERE al.name = ?
    GROUP BY al.id
    ''', (album,))
    return self.c.fetchall()[0]['artist']

  def song_of_path(self,path):
    self.c.execute('''
      SELECT  s.track  AS track
           ,  s.title  AS title
           ,  s.path   AS path
           , al.name   AS album
           ,  s.length AS length
           , ar.name   AS artist
        FROM song s
             JOIN album  al ON s.album_id  = al.id
             JOIN artist ar ON s.artist_id = ar.id
       WHERE s.path = ?
    GROUP BY s.id
    ORDER BY s.track ASC
           , s.id ASC
    ''', (path,))
    return self.c.fetchall()

  def get_songs(self,album):
    self.c.execute('''
      SELECT s.track AS track
           , s.title AS title
           , s.path  AS path
        FROM song s
             JOIN album  al ON s.album_id  = al.id
       WHERE al.name = ?
    GROUP BY s.id
    ORDER BY s.track ASC
           , s.id ASC
    ''', (album,))
    return self.c.fetchall()

  def get_albums(self):
    self.c.execute('''
      SELECT al.name AS album
           , ar.name AS artist
           , al.art_path AS art_path
        FROM song s
             JOIN album  al ON s.album_id  = al.id
             JOIN artist ar ON s.artist_id = ar.id
    GROUP BY al.id
    ''')
    return self.c.fetchall()

  def get_album_art(self,album):
    self.c.execute('''
      SELECT aa.x AS x
           , aa.y AS y
        FROM album a
             JOIN album_art aa ON a.id = aa.album_id
       WHERE a.name = ?
    ''', (album,))
    return self.c.fetchall()

  def get_theme_sprite(self,theme,sprite):
    self.c.execute('''
      SELECT *
        FROM theme_sprite ts
             JOIN theme_sprite_type tst ON ts.theme_id = tst.id
       WHERE tst.name = ?
         AND ts.name  = ?
    ''', (theme,sprite))
    return self.c.fetchall()[0]

  def get_theme_path(self,theme):
    self.c.execute('''
      SELECT path
        FROM theme_sprite_type tst
       WHERE tst.name = ?
    ''', (theme,))
    return self.c.fetchall()[0]['path']

db = DB()
