#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#
#   Copyright (c) 2007 Jesse Guardiani <jesse@guardiani.us>
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation; either version 2 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
#   02111-1307, USA.
#
# FIXME: make a few OGGs and test with those.

import sqlite3,os,mutagen,mutagen.easyid3,urllib,string,math,pygame,sys,time
import pygtk,gtk,gobject
from pygame.locals import *
import globals,prefs
from db      import db      as db
from theme   import theme   as theme


class DB():
  path = None
  con = None
  c = None

  def __init__(self,path):
    self.path = path
    try:
      ''' the alternative is to either drop each table and vacuume the DB,
      or else somehow diff the DB and only commit changes. Keep it simple
      for now. '''
      os.unlink(self.path)
    except:
      print "nothing to unlink"
    self.con = sqlite3.connect(self.path)
    self.con.row_factory = sqlite3.Row
    self.c   = self.con.cursor()
    self.c.execute('PRAGMA synchronous = OFF;')
    self.create_db()

  def _get_generic_id(self,table,name):
    valid_table_l = ['artist','album']
    if not table in valid_table_l:
      assert None # paranoia
    for i in [0,1]: #only two iterations. If the first fails, the second should always succeed!
      self.c.execute('SELECT id FROM '+table+' WHERE name=?',(name,))
      for row in self.c:
        return row['id']
      # no rows if we made it here, so insert one
      self.c.execute('INSERT INTO '+table+' (name) VALUES (?)',(name,))
    assert None # Should never ever make it here, so blow up if we do

  def create_db(self):
    self.c.execute('''
      CREATE TABLE album (
        id       INTEGER NOT NULL UNIQUE PRIMARY KEY AUTOINCREMENT,
        name     VARCHAR NOT NULL,
        art_path VARCHAR
      )
    ''')  
    self.c.execute('''
      CREATE INDEX album_name ON album (name)
    ''')
    
    self.c.execute('''
      CREATE TABLE artist (
        id   INTEGER NOT NULL UNIQUE PRIMARY KEY AUTOINCREMENT,
        name VARCHAR NOT NULL
      )
    ''')
    self.c.execute('''
      CREATE INDEX artist_name ON artist (name)
    ''')
    
    self.c.execute('''
      CREATE TABLE song (
        id        INTEGER NOT NULL UNIQUE PRIMARY KEY AUTOINCREMENT,
        track     INTEGER,
        title     VARCHAR NOT NULL,
        length    REAL    NOT NULL,
        album_id  INTEGER NOT NULL,
        artist_id INTEGER NOT NULL,
        path      VARCHAR NOT NULL
      )
    ''')
    self.c.execute('''
      CREATE INDEX song_title     ON song (title)
    ''')
    self.c.execute('''
      CREATE INDEX song_album_id  ON song (album_id)
    ''')
    self.c.execute('''
      CREATE INDEX song_artist_id ON song (artist_id)
    ''')
    self.c.execute('''
      CREATE INDEX song_path ON song (path)
    ''')

    self.c.execute('''
      CREATE TABLE version (
        value     INTEGER NOT NULL
      )
    ''')
    self.c.execute('''
      INSERT INTO version (value) VALUES (1)
    ''')

    self.c.execute('''
      CREATE TABLE album_art (
        id        INTEGER NOT NULL UNIQUE PRIMARY KEY AUTOINCREMENT,
        album_id  INTEGER NOT NULL,
        x         INTEGER NOT NULL,
        y         INTEGER NOT NULL
      )
    ''')

    self.c.execute('''
      CREATE TABLE theme_sprite_type (
        id        INTEGER NOT NULL UNIQUE PRIMARY KEY AUTOINCREMENT,
        name      VARCHAR NOT NULL,
        path      VARCHAR NOT NULL
      )
    ''')

    self.c.execute('''
      INSERT INTO theme_sprite_type (name,path) VALUES ('default','data/sprites.png')
    ''')

    self.c.execute('''
      CREATE TABLE theme_sprite (
        id        INTEGER NOT NULL UNIQUE PRIMARY KEY AUTOINCREMENT,
        theme_id  INTEGER NOT NULL,
        name      VARCHAR NOT NULL,
        x         INTEGER NOT NULL,
        y         INTEGER NOT NULL,
        w         INTEGER NOT NULL,
        h         INTEGER NOT NULL
      )
    ''')

    self.c.execute('''
      CREATE INDEX theme_sprite_theme_id_and_name ON theme_sprite (theme_id,name)
    ''')

    default_theme_sprite_list = [
      {'name':'text_bg_sel'   , 'x':  0, 'y':651, 'w':387, 'h': 79},
      {'name':'text_bg'       , 'x':  0, 'y':570, 'w':387, 'h': 79},
      {'name':'a2dp_on'       , 'x':176, 'y':488, 'w': 31, 'h': 43},
      {'name':'a2dp_off'      , 'x':208, 'y':488, 'w': 31, 'h': 43},
      {'name':'back_button'   , 'x':439, 'y':489, 'w': 39, 'h': 46},
      {'name':'exit_button'   , 'x':479, 'y':489, 'w': 39, 'h': 46},
      {'name':'background'    , 'x':  0, 'y':  0, 'w':800, 'h':480},
      {'name':'album_art_bg'  , 'x':703, 'y':490, 'w':125, 'h':160},
      {'name':'pause'         , 'x':371, 'y':489, 'w': 21, 'h': 39},
      {'name':'play'          , 'x':415, 'y':489, 'w': 21, 'h': 39},
      {'name':'next'          , 'x':371, 'y':531, 'w': 21, 'h': 31},
      {'name':'prev'          , 'x':393, 'y':531, 'w': 21, 'h': 31},
      {'name':'volume_bar_sel', 'x':157, 'y':521, 'w': 18, 'h': 11},
      {'name':'volume_bar'    , 'x':157, 'y':533, 'w': 18, 'h': 11},
      ]
    for row in default_theme_sprite_list:
      self.c.execute('''
        INSERT INTO theme_sprite (theme_id,name,x,y,w,h) VALUES (1,?,?,?,?,?)
        ''', (row['name'],row['x'],row['y'],row['w'],row['h']))

  def get_album_id(self,album):
    return self._get_generic_id('album',album)
  
  def get_artist_id(self,artist):
    return self._get_generic_id('artist',artist)
    
  def insert_song(self,track,title,artist,album,length,path):
    #print 'insert_song():\n\ttitle=%s\n\tartist=%s\n\talbum=%s\n\tlength=%s\n\tpath=%s' % \
    #    (title.encode('ascii','ignore'),artist.encode('ascii','ignore'),album.encode('ascii','ignore'),length,path.encode('ascii','ignore'))
    album_id  = self.get_album_id(album)
    artist_id = self.get_artist_id(artist)
    self.c.execute('''
      INSERT INTO song (track,title,length,album_id,artist_id,path) VALUES (?,?,?,?,?,?)
    ''',(track,title,length,album_id,artist_id,path))

  def get_album_paths(self):
    self.c.execute('''
      SELECT a.name AS album
           , s.path AS path
        FROM song s
             JOIN album a ON s.album_id=a.id
    GROUP BY s.album_id
    ''')
    return self.c.fetchall()

  def set_album_art_path(self,album,path):
    album_id  = self.get_album_id(album)
    self.c.execute('UPDATE album SET art_path = ? WHERE id = ?',(path,album_id))

  def insert_album_art(self,album,x,y):
    album_id  = self.get_album_id(album)
    self.c.execute('''
      INSERT INTO album_art (album_id,x,y) VALUES (?,?,?)
    ''',(album_id,x,y))

  def is_compilation(self,album):
    ''' Does this album have more than one artist? '''
    album_id  = self.get_album_id(album)
    self.c.execute('''
      SELECT s1.album_id
           , s1.artist_id
           , s2.artist_id
        FROM song s1
             JOIN song s2 ON s1.album_id = s2.album_id
       WHERE s1.artist_id != s2.artist_id
         AND s1.album_id = ?
    GROUP BY s1.album_id
    ''', (album_id,))
    for row in self.c:
      return 1
    return None

  def artist_of_album(self,album):
    ''' return list of all artists associated with a given album '''
    artist_a = []
    album_id  = self.get_album_id(album)
    self.c.execute('''
      SELECT a.name AS name
        FROM song s
             JOIN artist a ON s.artist_id = a.id
       WHERE s.album_id = ?
    GROUP BY a.id
    ''', (album_id,))
    for row in self.c:
      artist_a.append(row['name'])
    return artist_a


class Art():
  def __init__(self,name,width,height,art_path):
    self.name     = name
    self.width    = width
    self.height   = height
    self.art_path = art_path
    self.image    = pygame.Surface((self.width,self.height),0,theme.surface)
    theme.blit(self.image,(0,0),'album_art_bg')
    self.__load_image()

  def __load_image(self):
    print "art_path: " + self.art_path
    image = pygame.image.load(self.art_path)
    image.set_colorkey((255,0,0), pygame.RLEACCEL)

    if not image.get_width()==self.width:
      image=pygame.transform.scale(image, (self.width,128))
    self.image.blit(image, (0,0))

    #Reflection
    reflection=pygame.transform.flip(image, 0, 1)
    reflection.set_alpha(60)
    self.image.blit(reflection, (0,125))

    #artist name in the reflection space
    globals.gprint(self.name,self.image,(2, 145),16,False)

    self.rect  = self.image.get_rect()


class NewSongProcessor:
  # FIXME: add support for ogg
  ignore_dir_l = ['.','..','debian','maps','dev','proc','sys']
  ext_l = ['.mp3']
  data_dir = None
  update_func = None
  tick_func = None
  last_audioscrobbler_query = 0
  
  def __init__(self,DB,data_dir,update_func,tick_func):
    self.DB = DB
    self.data_dir = data_dir
    self.update_func = update_func
    self.tick_func = tick_func
    self.tot_filecount = 0
    self.filecount = 0

  def _get_mut_val(self,mut,name):
    ''' Sometimes we'll have a valid ID3 tag, but missing information.
    This method is responsible for how we handle these cases. '''
    try:
      if name == 'tracknumber':
        # usually track is '3', but sometimes '3/9'. deal with it.
        return mut[name][0].split("/")[0]
      else:
        return mut[name][0]
    except:
      if name == 'tracknumber':
        #print "WARNING: no %s tag, will use None: %s" % (name,mut.filename)
        return None
      else:
        print "WARNING: no %s tag, will use UNKNOWN: %s" % (name,mut.filename)
        return 'UNKNOWN'

  def _fix_net_search_string(self, s):
    s.replace("_"," ")
    s.replace(":"," ")
    s.replace("?"," ")
    s.replace("/"," ")
    s.replace("-"," ")
    s.replace("~"," ")
    s.replace("["," ")
    s.replace("]"," ")
    s.replace("(1)","")
    s.replace("(2)","")
    s.replace("(3)","")
    s.replace("(4)","")
    return s

  def recurse(self,path,depth=0,scout=False):
    ''' This is the primary directory tree walking method.
    It calls self.add_file() as it descends. '''
    try:
      if depth == 0:
        if scout: self.tot_filecount = 0
        self.filecount = 0
      depth+=1
      if (os.path.exists(path) and depth < 6):
        for fn in os.listdir(path):
          if fn.lower() in self.ignore_dir_l: # ignore these dirs
            continue

          cur_path = os.path.join(path,fn)
          #print "cur_path='"+cur_path+"'";
          (rootfn,ext) = os.path.splitext(fn)
          if ext.lower() in self.ext_l:
            self.filecount = self.filecount + 1
            if self.update_func: self.update_func(fn)
            if scout:
              self.tot_filecount = self.tot_filecount + 1
              if self.tick_func: self.tick_func()
            else:
              self.add_file(cur_path)
              if self.tick_func: self.tick_func(self.filecount * 100 / self.tot_filecount)

          elif os.path.isdir(cur_path):
            self.recurse(cur_path,depth,scout)
    except OSError, message:
      print "OSError:", message

  def add_file(self,path):
    ''' This method is responsible for parsing ID3 headers 
    and sending ID3 info off to the DB class for storage '''
    try:
      mut = mutagen.File(path)
      length = mut.info.length
      if isinstance(mut, mutagen.mp3.MP3): # special mutagen case for MP3s
        mut = mutagen.easyid3.EasyID3(path)
    except:
      print "ERROR: invalid or missing id3 header, skipping: ", path
      return
    self.DB.insert_song(
      self._get_mut_val(mut,'tracknumber'),
      self._get_mut_val(mut,'title'),
      self._get_mut_val(mut,'artist'),
      self._get_mut_val(mut,'album'),
      length,
      path,
    );

  def get_album_covers(self,album_cache_fn):
    width,height,x,y = 125,160,0,0
    list = self.DB.get_album_paths()
    num_albums = len(list)
    num_albums_per_side = int(math.ceil(math.sqrt(num_albums)))
    cache_image = pygame.Surface((width * num_albums_per_side,height * num_albums_per_side))

    c = 0
    for row in list:
      c = c + 1
      dir   = os.path.dirname(row['path'])
      album = row['album']

      if self.update_func:
        self.update_func(album)
        if self.tick_func: self.tick_func(c * 100 / len(list))

      art_path = self.get_existing_cover(dir)
      if not art_path: art_path = self.get_net_album_cover(album,dir)
      if not art_path: art_path = "data/UNKNOWN_UNKNOWN.jpg"

      self.DB.set_album_art_path(album,art_path)
      album_art = Art(album,width,height,art_path)
      self.DB.insert_album_art(album,x,y)
      cache_image.blit(album_art.image, (x,y),album_art.rect)
      x = x + width
      if x > cache_image.get_rect().width - width:
        x = 0
        y = y + height
    pygame.image.save(cache_image,album_cache_fn)

  def get_existing_cover(self,dir):
    for fn in ('folder.jpg','cover.jpg'):
      path = os.path.join(dir,fn)
      if os.path.exists(path):
        return path
    return None # nothing found

  def get_net_album_cover(self, album, dir):
    cover_dir = os.path.join(self.data_dir,'covers')
    if not os.path.exists(cover_dir): os.mkdir(cover_dir)
    fn=os.path.join(cover_dir,album.encode('ascii','ignore')+'.jpg')
    if os.path.exists(fn):
      return fn # file already exists...

    searchString='"'+album+'"'
    searchAltString=album
    artist = self.DB.artist_of_album(album)[0]
    
    if fn.find("NOARTIST") != -1 or fn.find("UNKNOWN") != -1: # ignore these
      return False

    if not self.DB.is_compilation(album):
      searchString=searchString+ ' '+ artist
      searchAltString=searchAltString+ ' OR '+ artist

    if self.retrieve_image_from_AudioScrobbler(artist, album, fn):
      return fn
    if self.retrieve_image_from_MSN(searchString + " (cd OR cover)",fn):
      return fn
    if self.retrieve_image_from_MSN(searchAltString + " (cd OR cover)",fn):
      return fn
    if self.retrieve_image_from_Yahoo(string.replace(artist+" "+ album," "," OR ")+ " OR cd",fn):
      return fn
    return False

  def retrieve_image_from_Yahoo(self, search_string, filename):
    print "Yahoo DL: "+filename.encode('ascii','ignore')
    urladdress="http://images.search.yahoo.com/search/images?"
    imageSearch = self._fix_net_search_string(search_string)
    params = {'p':imageSearch.encode('ascii', 'ignore')}

    txdata = urllib.urlencode(params)
    urladdress=urladdress+txdata+"&ei=UTF-8"
    #print "opening" + urladdress
    found=False
    try:
        sock = urllib.urlopen(urladdress)
        htmlSource = sock.read()
    except:
        print "unable to get socket"
        print sys.exc_info()[0]
    try:
        imagelocationStart=htmlSource.find('Go to fullsize image')+28
        if (imagelocationStart>28):
            imagelocationend=htmlSource.find('"',imagelocationStart)
            imagelocation=htmlSource[imagelocationStart:imagelocationend]
            urllib.urlretrieve(imagelocation, filename)
            found=True
        else:
            print "invalid image..."

        sock.close()
        return found
    except:
        print "Problem saving file :" + filename
        print sys.exc_info()[0]
        return False

  def retrieve_image_from_MSN(self, search_string, filename):
    print "MSN DL: "+filename.encode('ascii','ignore')
    urladdress="http://search.msn.com/images/results.aspx?"
    imageSearch = self._fix_net_search_string(search_string)
    params = {'q':imageSearch.encode('ascii', 'ignore')}

    txdata = urllib.urlencode(params)
    urladdress=urladdress+txdata+"&FORM=QBIR"
    #print "opening" + urladdress
    found=False
    try:
        sock = urllib.urlopen(urladdress)
        htmlSource = sock.read()
    except:
        print "unable to get socket"
        print sys.exc_info()[0]
    try:
        imagelocationStart=htmlSource.find('thumbnail')-52
        imagelocationStart=htmlSource.find('src="',imagelocationStart)+5
        #print imagelocationStart
        if (imagelocationStart>28):
            imagelocationend=htmlSource.find('"',imagelocationStart)
            imagelocation=htmlSource[imagelocationStart:imagelocationend]
            urllib.urlretrieve(imagelocation, filename)
            found=True
        else:
            print "invalid image..."

        sock.close()
        return found
    except:
        print "Problem saving file :" + filename
        print sys.exc_info()[0]
        return False

  def _encode_string(self, str):
#    print "to encode: "+str
    encoded = urllib.urlencode({'dummy':str})
    result = encoded[encoded.find('dummy=')+6:]
#    print "encoded: "+result
    return result

  def retrieve_image_from_AudioScrobbler(self, artist, album, filename):
    ti = int(time.time())

    # no more than one query per sec
    if (self.last_audioscrobbler_query>0) and (self.last_audioscrobbler_query>=ti):
      print "flood protection: skip AS"
      return False

    self.last_audioscrobbler_query = ti

    try:
      print "AS DL: "+filename.encode('ascii','ignore')
      urladdress="http://ws.audioscrobbler.com/1.0/album/"
      artist = self._fix_net_search_string(artist)
      album = self._fix_net_search_string(album)
      urladdress=urladdress + self._encode_string(artist) + "/" + self._encode_string(album) + "/info.xml"
#       print "opening " + urladdress
    except:
      print "AS: exception in url builder"
      return False

    found=False
    try:
        sock = urllib.urlopen(urladdress)
        htmlSource = sock.read()
    except:
        print "unable to get socket"
        print sys.exc_info()[0]
    try:
        imagelocationStart=htmlSource.find('<coverart>') + 10
        imagelocationStart=htmlSource.find('<medium>', imagelocationStart) + 8
        #print imagelocationStart
        if (imagelocationStart>19):
            imagelocationend=htmlSource.find('</medium>',imagelocationStart)
            imagelocation=htmlSource[imagelocationStart:imagelocationend]
            if imagelocation.find('/noimage/')>0:
              print 'album cover not found in AS'
            else:
              urllib.urlretrieve(imagelocation, filename)
              found=True
        else:
            print "artist/album pair not found in AS"

        sock.close()
        return found
    except:
        print "Problem saving file :" + filename
        print sys.exc_info()[0]
        return False


class Scanner():
  global theme

  def quit(self, data=None):
    sys.exit(0)

  def are_you_sure_dialog(self):
    dialog = gtk.Dialog("Scan for music", self.window, 0, (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT, gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
    dialog.set_default_response(gtk.RESPONSE_ACCEPT)
    
    label = gtk.Label("    Would you like to scan your device for music?    ") #fixme: ugly
    dialog.vbox.pack_start(label, False, True, 15)
    
    dialog.vbox.show_all()
    if dialog.run() != gtk.RESPONSE_ACCEPT:
      dialog.destroy()
      self.quit()
    
    dialog.destroy()


  def completed_dialog(self):
    dialog = gtk.Dialog("Scan complete", self.window, 0, (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
    dialog.set_default_response(gtk.RESPONSE_ACCEPT)
    
    label = gtk.Label("   Succesfully finished scanning your music collection.   \n   Now run Kagu Media Player to listen.   ") #fixme: ugly
    dialog.vbox.pack_start(label, False, True, 15)

    dialog.vbox.show_all()
    dialog.run()
    dialog.destroy()


  def __init__(self):
    global theme
    
    if globals.ISMAEMO:
      import hildon
      self.app = hildon.Program()
      self.window = hildon.Window()
    else:
      self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)

    self.window.set_title("kagu scanner")
    self.window.connect("destroy", self.quit)

    if globals.ISMAEMO: self.app.add_window(self.window)

    self.pbar = gtk.ProgressBar()
    self.pbarlabel = gtk.Label("Ready to scan...")
    self.commentlabel = gtk.Label()

    vbox = gtk.VBox(False, 0)
    vbox.pack_start(self.pbarlabel, False, False, 0)
    vbox.pack_start(self.pbar, False, False, 0)
    vbox.pack_start(self.commentlabel, False, False, 0)
    self.window.add(vbox)

    self.window.show_all()
    self.scan_tick()

    self.are_you_sure_dialog()
    self.pbarlabel.set_text("Preparing...")
    self.scan_tick()
    
    self.path_a = globals.get_path_list()
    self.db_dir = globals.calc_db_dir()
    self.mydb = DB(os.path.join(self.db_dir, 'kagu.db'))
    self.mydb.con.commit()

    db.reconnect()

    self.myprefs = prefs.Prefs()
    theme.set_theme(self.myprefs.get('theme'),False)
    globals.MAINFONTCOLOR   = theme.surface.get_at((93,512))
    globals.MAINFONTBGCOLOR = theme.surface.get_at((40,582))

    self.sp=NewSongProcessor(self.mydb, self.db_dir, self.scan_update, self.scan_tick)
    self.pbarlabel.set_text("Scanning...")
    self.scan_tick()

    os.nice(5)

    for path in self.path_a: # scan for music
      if os.path.exists(path):
        self.pbarlabel.set_text("Looking for files in " + path)
        self.sp.recurse(path,0,True)
        self.pbarlabel.set_text("Reading ID3 tags from " + path)
        self.sp.recurse(path)
        self.mydb.con.commit()

    self.pbarlabel.set_text("Downloading album covers...")
    self.scan_tick()
    self.sp.get_album_covers(os.path.join(self.db_dir, 'album_cache.tga'))

    self.scan_update("")
    self.pbarlabel.set_text("Completed")
    self.scan_tick()
    self.mydb.con.commit()
    self.completed_dialog()
    self.quit()


  def scan_update(self, fn):
    if fn:
      self.commentlabel.set_text(fn)
      self.pbar.pulse()    


  def scan_tick(self,percent=None):
    if percent: self.pbar.set_fraction(percent/100.0)
    while gtk.events_pending():
      gtk.main_iteration()


def init_pygame():
  SCREENRECT = Rect(0, 0, 800, 480)
  os.environ["SDL_VIDEO_X11_WMCLASS"]="kaguscanner"
  pygame.init()

def main():
  init_pygame()
  Scanner()

if __name__ == '__main__': main()
