#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#
#   Copyright (c) 2007 Jesse Guardiani <jesse@guardiani.us>
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation; either version 2 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
#   02111-1307, USA.
#

import os, pygame, time, sys, gc
from pygame.locals import *
import gtk, remote
import globals


def init_pygame():
  SCREENRECT = Rect(0, 0, 800, 480)

  os.environ["SDL_VIDEO_X11_WMCLASS"]="kagu"

  pygame.init()
  pygame.display.set_caption('kagu')
  pygame.mixer.quit() # we don't want pygame hogging the audio device

  # set the display mode
  winstyle = FULLSCREEN
  if not globals.ISMAEMO: winstyle = 1

  bestdepth = pygame.display.mode_ok(SCREENRECT.size, winstyle, 16)
  if not globals.ISMAEMO: bestdepth = pygame.display.mode_ok(SCREENRECT.size, winstyle, 32)

  screen = pygame.display.set_mode(SCREENRECT.size, winstyle, bestdepth)
  if globals.ISMAEMO: pygame.mouse.set_visible(False)
  return screen


def main():
  if remote.remote(sys.argv): return

  dbfile = os.path.join(globals.calc_db_dir(), 'kagu.db')
  dbexists = os.path.exists(dbfile)
  if (not dbexists) or (dbexists and os.path.getsize(dbfile)==0):
    print "kagu.db not found - launching scanner"
    os.popen("./kagu-scanner.py")
    sys.exit(0)

  from db      import db as db
  from manager import manager as manager
  from theme   import theme   as theme
  import widgets, views

  #
  # Init PyGame
  #
  screen = init_pygame()

  globals.albumcache = pygame.image.load(os.path.join(db.get_db_dir(),"album_cache.tga")).convert()

  #
  # Start up the main logic object
  #
  manager.start(screen)

  #
  # Event Loop
  #
  clock = pygame.time.Clock()
  idle_start = 0

  while 1:
    clock.tick(30)

    events = pygame.event.get()
    if not events and manager.idle_seconds>1: time.sleep(0.1)
    if manager.is_extended_idle():
      print "idle"
      while not events and manager.is_extended_idle():
        # this is an optimized event loop designed to:
        # 1.) service manager.player
        # 2.) check for new pygame events with low latency and
        #     low CPU because we are not redrawing the screen
        time.sleep(0.1)
        events = pygame.event.get()
        gtk.main_iteration(block=False)
    if events: manager.wake_up()
    for event in events:
      if event.type == QUIT:
        manager.quit()
        return
      if event.type == KEYDOWN and event.key == K_F4:
        pygame.display.toggle_fullscreen()
        break
      manager.handle_event(event)

    manager.update()

    # draw everything
    manager.clear()
    manager.draw()
    pygame.display.flip()
    gtk.main_iteration(block=False)


if __name__ == '__main__': main()
