#!/usr/bin/env python

import os, ConfigParser
from manager import manager as manager

PREFSFILE = os.path.expanduser("~/.kagu/preferences")

#
#  Implements storage and retrieval of GMP preferences.
#
class Prefs:
  
  parser = None
  
  defaults = { 
    "continuous" : "True",
    "random"     : "False",
    "repeat"     : "False",
    "sort"       : "0",
    "volume"     : "70",
    "theme"      : "default",
  }
  
  #
  #  Instantiates a new Prefs with the options from STORE.
  #
  def __init__(self):
    self.parser = ConfigParser.SafeConfigParser(self.defaults)
    
    try:
      self.parser.read([PREFSFILE,])
    except StandardError:
      return
    
  #
  #  A convenience method for retrieving an option.
  #
  def get(self, option):
    return self.parser.get("DEFAULT", option)
    
  #
  #  A convenience method for retrieving an integer option.
  #
  def getInt(self, option):
    return int(self.parser.get("DEFAULT", option))
    
  #
  #  A convenience method for retrieving a boolean option.
  #
  def getBool(self, option):
    return self.parser.get("DEFAULT", option) == "True"
    
  #
  #  A convenience method for setting an option.
  #
  def set(self, option, value):
    self.parser.set("DEFAULT", option, value) 
    
  #
  #  Writes prefs to the specified file, defaulting to PREFSFILE.
  #
  def save(self, prefsFile=PREFSFILE):
    self.set("continuous", str(manager.playlist.continuous))
    self.set("random", str(manager.playlist.random))
    self.set("repeat", str(manager.playlist.repeat))
    self.set("volume", str(manager.player.volume))
    
    try:
      self.parser.write(open(prefsFile, "w"))
    except StandardError:
      return
      
#End of file
